@@ -1,28 +1,34 @@
 /* Low level Unix child interface to ptrace, for GDB when running under Unix.
-   Copyright 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996 Free Software Foundation, Inc.
+   Copyright 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1998,
+   1999, 2000, 2001
+   Free Software Foundation, Inc.
 
-This file is part of GDB.
+   This file is part of GDB.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
 #include "defs.h"
 #include "frame.h"
 #include "inferior.h"
 #include "target.h"
 #include "gdb_string.h"
-#include "wait.h"
+#include "regcache.h"
+
+#include "gdb_wait.h"
+
 #include "command.h"
 
 #ifdef USG
@@ -30,17 +36,17 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #endif
 
 #include <sys/param.h>
-#include <sys/dir.h>
+#include "gdb_dirent.h"
 #include <signal.h>
 #include <sys/ioctl.h>
 
-#ifndef NO_PTRACE_H
-#ifdef PTRACE_IN_WRONG_PLACE
+#ifdef HAVE_PTRACE_H
 #include <ptrace.h>
 #else
+#ifdef HAVE_SYS_PTRACE_H
 #include <sys/ptrace.h>
 #endif
-#endif /* NO_PTRACE_H */
+#endif
 
 #if !defined (PT_READ_I)
 #define PT_READ_I	1	/* Read word from text space */
@@ -95,44 +101,132 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #endif /* KERNEL_U_ADDR_BSD.  */
 #endif /* !FETCH_INFERIOR_REGISTERS */
 
+#if !defined (CHILD_XFER_MEMORY)
+static void udot_info (char *, int);
+#endif
+
+#if !defined (FETCH_INFERIOR_REGISTERS)
+static void fetch_register (int);
+static void store_register (int);
+#endif
+
+void _initialize_kernel_u_addr (void);
+void _initialize_infptrace (void);
 
+
 /* This function simply calls ptrace with the given arguments.  
    It exists so that all calls to ptrace are isolated in this 
    machine-dependent file. */
 int
-call_ptrace (request, pid, addr, data)
-     int request, pid;
-     PTRACE_ARG3_TYPE addr;
-     int data;
+call_ptrace (int request, int pid, PTRACE_ARG3_TYPE addr, int data)
 {
-  return ptrace (request, pid, addr, data
-#if defined (FIVE_ARG_PTRACE)
-		 /* Deal with HPUX 8.0 braindamage.  We never use the
-		    calls which require the fifth argument.  */
-		 , 0
+  int pt_status = 0;
+
+#if 0
+  int saved_errno;
+
+  printf ("call_ptrace(request=%d, pid=%d, addr=0x%x, data=0x%x)",
+	  request, pid, addr, data);
 #endif
-		 );
+#if defined(PT_SETTRC)
+  /* If the parent can be told to attach to us, try to do it.  */
+  if (request == PT_SETTRC)
+    {
+      errno = 0;
+#if !defined (FIVE_ARG_PTRACE)
+      pt_status = ptrace (PT_SETTRC, pid, addr, data);
+#else
+      /* Deal with HPUX 8.0 braindamage.  We never use the
+         calls which require the fifth argument.  */
+      pt_status = ptrace (PT_SETTRC, pid, addr, data, 0);
+#endif
+      if (errno)
+	perror_with_name ("ptrace");
+#if 0
+      printf (" = %d\n", pt_status);
+#endif
+      if (pt_status < 0)
+	return pt_status;
+      else
+	return parent_attach_all (pid, addr, data);
+    }
+#endif
+
+#if defined(PT_CONTIN1)
+  /* On HPUX, PT_CONTIN1 is a form of continue that preserves pending
+     signals.  If it's available, use it.  */
+  if (request == PT_CONTINUE)
+    request = PT_CONTIN1;
+#endif
+
+#if defined(PT_SINGLE1)
+  /* On HPUX, PT_SINGLE1 is a form of step that preserves pending
+     signals.  If it's available, use it.  */
+  if (request == PT_STEP)
+    request = PT_SINGLE1;
+#endif
+
+#if 0
+  saved_errno = errno;
+  errno = 0;
+#endif
+#if !defined (FIVE_ARG_PTRACE)
+  pt_status = ptrace (request, pid, addr, data);
+#else
+  /* Deal with HPUX 8.0 braindamage.  We never use the
+     calls which require the fifth argument.  */
+  pt_status = ptrace (request, pid, addr, data, 0);
+#endif
+
+#if 0
+  if (errno)
+    printf (" [errno = %d]", errno);
+
+  errno = saved_errno;
+  printf (" = 0x%x\n", pt_status);
+#endif
+  return pt_status;
 }
 
+
 #if defined (DEBUG_PTRACE) || defined (FIVE_ARG_PTRACE)
 /* For the rest of the file, use an extra level of indirection */
 /* This lets us breakpoint usefully on call_ptrace. */
 #define ptrace call_ptrace
 #endif
 
-void
-kill_inferior ()
+/* Wait for a process to finish, possibly running a target-specific
+   hook before returning.  */
+
+int
+ptrace_wait (ptid_t ptid, int *status)
 {
-  if (inferior_pid == 0)
+  int wstate;
+
+  wstate = wait (status);
+  target_post_wait (pid_to_ptid (wstate), *status);
+  return wstate;
+}
+
+void
+kill_inferior (void)
+{
+  int status;
+  int pid =  PIDGET (inferior_ptid);
+
+  if (pid == 0)
     return;
-  /* ptrace PT_KILL only works if process is stopped!!!  So stop it with
-     a real signal first, if we can.  FIXME: This is bogus.  When the inferior
-     is not stopped, GDB should just be waiting for it.  Either the following
-     line is unecessary, or there is some problem elsewhere in GDB which
-     causes us to get here when the inferior is not stopped.  */
-  kill (inferior_pid, SIGKILL);
-  ptrace (PT_KILL, inferior_pid, (PTRACE_ARG3_TYPE) 0, 0);
-  wait ((int *)0);
+
+  /* This once used to call "kill" to kill the inferior just in case
+     the inferior was still running.  As others have noted in the past
+     (kingdon) there shouldn't be any way to get here if the inferior
+     is still running -- else there's a major problem elsewere in gdb
+     and it needs to be fixed.
+
+     The kill call causes problems under hpux10, so it's been removed;
+     if this causes problems we'll deal with them as they arise.  */
+  ptrace (PT_KILL, pid, (PTRACE_ARG3_TYPE) 0, 0);
+  ptrace_wait (null_ptid, &status);
   target_mourn_inferior ();
 }
 
@@ -143,18 +237,17 @@ kill_inferior ()
    If SIGNAL is nonzero, give it that signal.  */
 
 void
-child_resume (pid, step, signal)
-     int pid;
-     int step;
-     enum target_signal signal;
+child_resume (ptid_t ptid, int step, enum target_signal signal)
 {
+  int pid = PIDGET (ptid);
+
   errno = 0;
 
   if (pid == -1)
     /* Resume all threads.  */
     /* I think this only gets used in the non-threaded case, where "resume
-       all threads" and "resume inferior_pid" are the same.  */
-    pid = inferior_pid;
+       all threads" and "resume inferior_ptid" are the same.  */
+    pid = PIDGET (inferior_ptid);
 
   /* An address of (PTRACE_ARG3_TYPE)1 tells ptrace to continue from where
      it was.  (If GDB wanted it to start some other way, we have already
@@ -166,23 +259,29 @@ child_resume (pid, step, signal)
      instructions), so we don't have to worry about that here.  */
 
   if (step)
-    ptrace (PT_STEP,     pid, (PTRACE_ARG3_TYPE) 1,
-	    target_signal_to_host (signal));
+    {
+      if (SOFTWARE_SINGLE_STEP_P ())
+	internal_error (__FILE__, __LINE__, "failed internal consistency check");		/* Make sure this doesn't happen. */
+      else
+	ptrace (PT_STEP, pid, (PTRACE_ARG3_TYPE) 1,
+		target_signal_to_host (signal));
+    }
   else
     ptrace (PT_CONTINUE, pid, (PTRACE_ARG3_TYPE) 1,
 	    target_signal_to_host (signal));
 
   if (errno)
-    perror_with_name ("ptrace");
+    {
+      perror_with_name ("ptrace");
+    }
 }
 #endif /* CHILD_RESUME */
-
 
+
 #ifdef ATTACH_DETACH
 /* Start debugging the process whose number is PID.  */
 int
-attach (pid)
-     int pid;
+attach (int pid)
 {
   errno = 0;
   ptrace (PT_ATTACH, pid, (PTRACE_ARG3_TYPE) 0, 0);
@@ -197,11 +296,11 @@ attach (pid)
    SIGNAL = 0 means just continue it.  */
 
 void
-detach (signal)
-     int signal;
+detach (int signal)
 {
   errno = 0;
-  ptrace (PT_DETACH, inferior_pid, (PTRACE_ARG3_TYPE) 1, signal);
+  ptrace (PT_DETACH, PIDGET (inferior_ptid), (PTRACE_ARG3_TYPE) 1,
+          signal);
   if (errno)
     perror_with_name ("ptrace");
   attach_flag = 0;
@@ -221,7 +320,7 @@ CORE_ADDR kernel_u_addr;
 #endif /* KERNEL_U_ADDR_BSD.  */
 
 void
-_initialize_kernel_u_addr ()
+_initialize_kernel_u_addr (void)
 {
 #if defined (KERNEL_U_ADDR_BSD) && !defined (FETCH_INFERIOR_REGISTERS)
   struct nlist names[2];
@@ -231,7 +330,8 @@ _initialize_kernel_u_addr ()
   if (nlist ("/vmunix", names) == 0)
     kernel_u_addr = names[0].n_value;
   else
-    fatal ("Unable to get kernel u area address.");
+    internal_error (__FILE__, __LINE__,
+		    "Unable to get kernel u area address.");
 #endif /* KERNEL_U_ADDR_BSD.  */
 }
 
@@ -244,30 +344,23 @@ _initialize_kernel_u_addr ()
 /* U_REGS_OFFSET is the offset of the registers within the u area.  */
 #if !defined (U_REGS_OFFSET)
 #define U_REGS_OFFSET \
-  ptrace (PT_READ_U, inferior_pid, \
+  ptrace (PT_READ_U, PIDGET (inferior_ptid), \
 	  (PTRACE_ARG3_TYPE) (offsetof (struct user, u_ar0)), 0) \
     - KERNEL_U_ADDR
 #endif
 
-/* Registers we shouldn't try to fetch.  */
-#if !defined (CANNOT_FETCH_REGISTER)
-#define CANNOT_FETCH_REGISTER(regno) 0
-#endif
-
 /* Fetch one register.  */
 
 static void
-fetch_register (regno)
-     int regno;
+fetch_register (int regno)
 {
   /* This isn't really an address.  But ptrace thinks of it as one.  */
   CORE_ADDR regaddr;
-  char buf[MAX_REGISTER_RAW_SIZE];
-  char mess[128];				/* For messages */
+  char mess[128];		/* For messages */
   register int i;
-
-  /* Offset of registers within the u area.  */
-  unsigned int offset;
+  unsigned int offset;		/* Offset of registers within the u area.  */
+  char buf[MAX_REGISTER_RAW_SIZE];
+  int tid;
 
   if (CANNOT_FETCH_REGISTER (regno))
     {
@@ -276,18 +369,23 @@ fetch_register (regno)
       return;
     }
 
+  /* Overload thread id onto process id */
+  if ((tid = TIDGET (inferior_ptid)) == 0)
+    tid = PIDGET (inferior_ptid);	/* no thread id, just use process id */
+
   offset = U_REGS_OFFSET;
 
   regaddr = register_addr (regno, offset);
   for (i = 0; i < REGISTER_RAW_SIZE (regno); i += sizeof (PTRACE_XFER_TYPE))
     {
       errno = 0;
-      *(PTRACE_XFER_TYPE *) &buf[i] = ptrace (PT_READ_U, inferior_pid,
-					      (PTRACE_ARG3_TYPE) regaddr, 0);
+      *(PTRACE_XFER_TYPE *) & buf[i] = ptrace (PT_READ_U, tid,
+					       (PTRACE_ARG3_TYPE) regaddr, 0);
       regaddr += sizeof (PTRACE_XFER_TYPE);
       if (errno != 0)
 	{
-	  sprintf (mess, "reading register %s (#%d)", reg_names[regno], regno);
+	  sprintf (mess, "reading register %s (#%d)", 
+		   REGISTER_NAME (regno), regno);
 	  perror_with_name (mess);
 	}
     }
@@ -295,80 +393,81 @@ fetch_register (regno)
 }
 
 
-/* Fetch all registers, or just one, from the child process.  */
+/* Fetch register values from the inferior.
+   If REGNO is negative, do this for all registers.
+   Otherwise, REGNO specifies which register (so we can save time). */
 
 void
-fetch_inferior_registers (regno)
-     int regno;
+fetch_inferior_registers (int regno)
 {
-  int numregs;
-
-  if (regno == -1)
+  if (regno >= 0)
     {
-      numregs = ARCH_NUM_REGS;
-      for (regno = 0; regno < numregs; regno++)
-        fetch_register (regno);
+      fetch_register (regno);
     }
   else
-    fetch_register (regno);
+    {
+      for (regno = 0; regno < NUM_REGS; regno++)
+	{
+	  fetch_register (regno);
+	}
+    }
 }
 
-/* Registers we shouldn't try to store.  */
-#if !defined (CANNOT_STORE_REGISTER)
-#define CANNOT_STORE_REGISTER(regno) 0
-#endif
+/* Store one register. */
 
-/* Store our register values back into the inferior.
-   If REGNO is -1, do this for all registers.
-   Otherwise, REGNO specifies which register (so we can save time).  */
-
-void
-store_inferior_registers (regno)
-     int regno;
+static void
+store_register (int regno)
 {
   /* This isn't really an address.  But ptrace thinks of it as one.  */
   CORE_ADDR regaddr;
-  char buf[80];
-  register int i, numregs;
+  char mess[128];		/* For messages */
+  register int i;
+  unsigned int offset;		/* Offset of registers within the u area.  */
+  int tid;
 
-  unsigned int offset = U_REGS_OFFSET;
+  if (CANNOT_STORE_REGISTER (regno))
+    {
+      return;
+    }
 
+  /* Overload thread id onto process id */
+  if ((tid = TIDGET (inferior_ptid)) == 0)
+    tid = PIDGET (inferior_ptid);	/* no thread id, just use process id */
+
+  offset = U_REGS_OFFSET;
+
+  regaddr = register_addr (regno, offset);
+  for (i = 0; i < REGISTER_RAW_SIZE (regno); i += sizeof (PTRACE_XFER_TYPE))
+    {
+      errno = 0;
+      ptrace (PT_WRITE_U, tid, (PTRACE_ARG3_TYPE) regaddr,
+	      *(PTRACE_XFER_TYPE *) & registers[REGISTER_BYTE (regno) + i]);
+      regaddr += sizeof (PTRACE_XFER_TYPE);
+      if (errno != 0)
+	{
+	  sprintf (mess, "writing register %s (#%d)", 
+		   REGISTER_NAME (regno), regno);
+	  perror_with_name (mess);
+	}
+    }
+}
+
+/* Store our register values back into the inferior.
+   If REGNO is negative, do this for all registers.
+   Otherwise, REGNO specifies which register (so we can save time).  */
+
+void
+store_inferior_registers (int regno)
+{
   if (regno >= 0)
     {
-      regaddr = register_addr (regno, offset);
-      for (i = 0; i < REGISTER_RAW_SIZE (regno); i += sizeof(PTRACE_XFER_TYPE))
-	{
-	  errno = 0;
-	  ptrace (PT_WRITE_U, inferior_pid, (PTRACE_ARG3_TYPE) regaddr,
-		  *(PTRACE_XFER_TYPE *) &registers[REGISTER_BYTE (regno) + i]);
-	  if (errno != 0)
-	    {
-	      sprintf (buf, "writing register number %d(%d)", regno, i);
-	      perror_with_name (buf);
-	    }
-	  regaddr += sizeof(PTRACE_XFER_TYPE);
-	}
+      store_register (regno);
     }
   else
     {
-      numregs = ARCH_NUM_REGS;
-      for (regno = 0; regno < numregs; regno++)
+      for (regno = 0; regno < NUM_REGS; regno++)
 	{
-	  if (CANNOT_STORE_REGISTER (regno))
-	    continue;
-	  regaddr = register_addr (regno, offset);
-	  for (i = 0; i < REGISTER_RAW_SIZE (regno); i += sizeof(PTRACE_XFER_TYPE))
-	    {
-	      errno = 0;
-	      ptrace (PT_WRITE_U, inferior_pid, (PTRACE_ARG3_TYPE) regaddr,
-		      *(PTRACE_XFER_TYPE *) &registers[REGISTER_BYTE (regno) + i]);
-	      if (errno != 0)
-		{
-		  sprintf (buf, "writing register number %d(%d)", regno, i);
-		  perror_with_name (buf);
-		}
-	      regaddr += sizeof(PTRACE_XFER_TYPE);
-	    }
+	  store_register (regno);
 	}
     }
 }
@@ -384,46 +483,44 @@ store_inferior_registers (regno)
 
 /* Copy LEN bytes to or from inferior's memory starting at MEMADDR
    to debugger memory starting at MYADDR.   Copy to inferior if
-   WRITE is nonzero.
-  
+   WRITE is nonzero.  TARGET is ignored.
+
    Returns the length copied, which is either the LEN argument or zero.
    This xfer function does not do partial moves, since child_ops
    doesn't allow memory operations to cross below us in the target stack
    anyway.  */
 
 int
-child_xfer_memory (memaddr, myaddr, len, write, target)
-     CORE_ADDR memaddr;
-     char *myaddr;
-     int len;
-     int write;
-     struct target_ops *target;		/* ignored */
+child_xfer_memory (CORE_ADDR memaddr, char *myaddr, int len, int write,
+		   struct mem_attrib *attrib ATTRIBUTE_UNUSED,
+		   struct target_ops *target)
 {
   register int i;
   /* Round starting address down to longword boundary.  */
-  register CORE_ADDR addr = memaddr & - sizeof (PTRACE_XFER_TYPE);
+  register CORE_ADDR addr = memaddr & -(CORE_ADDR) sizeof (PTRACE_XFER_TYPE);
   /* Round ending address up; get number of longwords that makes.  */
   register int count
-    = (((memaddr + len) - addr) + sizeof (PTRACE_XFER_TYPE) - 1)
-      / sizeof (PTRACE_XFER_TYPE);
+  = (((memaddr + len) - addr) + sizeof (PTRACE_XFER_TYPE) - 1)
+  / sizeof (PTRACE_XFER_TYPE);
   /* Allocate buffer of that many longwords.  */
   register PTRACE_XFER_TYPE *buffer
-    = (PTRACE_XFER_TYPE *) alloca (count * sizeof (PTRACE_XFER_TYPE));
+  = (PTRACE_XFER_TYPE *) alloca (count * sizeof (PTRACE_XFER_TYPE));
 
   if (write)
     {
       /* Fill start and end extra bytes of buffer with existing memory data.  */
 
-      if (addr != memaddr || len < (int) sizeof (PTRACE_XFER_TYPE)) {
-	/* Need part of initial word -- fetch it.  */
-        buffer[0] = ptrace (PT_READ_I, inferior_pid, (PTRACE_ARG3_TYPE) addr,
-			    0);
-      }
+      if (addr != memaddr || len < (int) sizeof (PTRACE_XFER_TYPE))
+	{
+	  /* Need part of initial word -- fetch it.  */
+	  buffer[0] = ptrace (PT_READ_I, PIDGET (inferior_ptid), 
+			      (PTRACE_ARG3_TYPE) addr, 0);
+	}
 
       if (count > 1)		/* FIXME, avoid if even boundary */
 	{
-	  buffer[count - 1]
-	    = ptrace (PT_READ_I, inferior_pid,
+	  buffer[count - 1] 
+	    = ptrace (PT_READ_I, PIDGET (inferior_ptid),
 		      ((PTRACE_ARG3_TYPE)
 		       (addr + (count - 1) * sizeof (PTRACE_XFER_TYPE))),
 		      0);
@@ -440,19 +537,22 @@ child_xfer_memory (memaddr, myaddr, len, write, target)
       for (i = 0; i < count; i++, addr += sizeof (PTRACE_XFER_TYPE))
 	{
 	  errno = 0;
-	  ptrace (PT_WRITE_D, inferior_pid, (PTRACE_ARG3_TYPE) addr,
-		  buffer[i]);
+	  ptrace (PT_WRITE_D, PIDGET (inferior_ptid), 
+		  (PTRACE_ARG3_TYPE) addr, buffer[i]);
 	  if (errno)
 	    {
 	      /* Using the appropriate one (I or D) is necessary for
-		 Gould NP1, at least.  */
+	         Gould NP1, at least.  */
 	      errno = 0;
-	      ptrace (PT_WRITE_I, inferior_pid, (PTRACE_ARG3_TYPE) addr,
-		      buffer[i]);
+	      ptrace (PT_WRITE_I, PIDGET (inferior_ptid), 
+		      (PTRACE_ARG3_TYPE) addr, buffer[i]);
 	    }
 	  if (errno)
 	    return 0;
 	}
+#ifdef CLEAR_INSN_CACHE
+      CLEAR_INSN_CACHE ();
+#endif
     }
   else
     {
@@ -460,7 +560,7 @@ child_xfer_memory (memaddr, myaddr, len, write, target)
       for (i = 0; i < count; i++, addr += sizeof (PTRACE_XFER_TYPE))
 	{
 	  errno = 0;
-	  buffer[i] = ptrace (PT_READ_I, inferior_pid,
+	  buffer[i] = ptrace (PT_READ_I, PIDGET (inferior_ptid),
 			      (PTRACE_ARG3_TYPE) addr, 0);
 	  if (errno)
 	    return 0;
@@ -474,19 +574,21 @@ child_xfer_memory (memaddr, myaddr, len, write, target)
     }
   return len;
 }
-
 
-static void
-udot_info ()
-{
-  int udot_off;		/* Offset into user struct */
-  int udot_val;		/* Value from user struct at udot_off */
-  char mess[128];	/* For messages */
 
-   if (!target_has_execution)
-     {
-       error ("The program is not being run.");
-     }
+static void
+udot_info (char *dummy1, int dummy2)
+{
+#if defined (KERNEL_U_SIZE)
+  int udot_off;			/* Offset into user struct */
+  int udot_val;			/* Value from user struct at udot_off */
+  char mess[128];		/* For messages */
+#endif
+
+  if (!target_has_execution)
+    {
+      error ("The program is not being run.");
+    }
 
 #if !defined (KERNEL_U_SIZE)
 
@@ -508,7 +610,7 @@ udot_info ()
 	    }
 	  printf_filtered ("%04x:", udot_off);
 	}
-      udot_val = ptrace (PT_READ_U, inferior_pid, (PTRACE_ARG3_TYPE) udot_off, 0);
+      udot_val = ptrace (PT_READ_U, PIDGET (inferior_ptid), (PTRACE_ARG3_TYPE) udot_off, 0);
       if (errno != 0)
 	{
 	  sprintf (mess, "\nreading user struct at offset 0x%x", udot_off);
@@ -522,10 +624,10 @@ udot_info ()
 #endif
 }
 #endif /* !defined (CHILD_XFER_MEMORY).  */
-
 
+
 void
-_initialize_infptrace ()
+_initialize_infptrace (void)
 {
 #if !defined (CHILD_XFER_MEMORY)
   add_info ("udot", udot_info,
