@@ -1,5 +1,6 @@
 /* Read HP PA/Risc object files for GDB.
-   Copyright 1991, 1992, 1996, 1999 Free Software Foundation, Inc.
+   Copyright 1991, 1992, 1994, 1995, 1996, 1998, 1999, 2000
+   Free Software Foundation, Inc.
    Written by Fred Fish at Cygnus Support.
 
    This file is part of GDB.
@@ -36,38 +37,29 @@
 
 /* Various things we might complain about... */
 
-static void
-som_symfile_init PARAMS ((struct objfile *));
+static void som_symfile_init (struct objfile *);
+
+static void som_new_init (struct objfile *);
+
+static void som_symfile_read (struct objfile *, int);
+
+static void som_symfile_finish (struct objfile *);
 
 static void
-som_new_init PARAMS ((struct objfile *));
+som_symtab_read (bfd *, struct objfile *, struct section_offsets *);
 
 static void
-som_symfile_read PARAMS ((struct objfile *, int));
-
-static void
-som_symfile_finish PARAMS ((struct objfile *));
-
-static void
-som_symtab_read PARAMS ((bfd *, struct objfile *,
-			 struct section_offsets *));
-
-static void
-som_symfile_offsets PARAMS ((struct objfile *, struct section_addr_info *));
+som_symfile_offsets (struct objfile *, struct section_addr_info *);
 
 /* FIXME: These should really be in a common header somewhere */
 
-extern void
-hpread_build_psymtabs PARAMS ((struct objfile *, int));
+extern void hpread_build_psymtabs (struct objfile *, int);
 
-extern void
-hpread_symfile_finish PARAMS ((struct objfile *));
+extern void hpread_symfile_finish (struct objfile *);
 
-extern void
-hpread_symfile_init PARAMS ((struct objfile *));
+extern void hpread_symfile_init (struct objfile *);
 
-extern void
-do_pxdb PARAMS ((bfd *));
+extern void do_pxdb (bfd *);
 
 /*
 
@@ -89,10 +81,8 @@ do_pxdb PARAMS ((bfd *));
  */
 
 static void
-som_symtab_read (abfd, objfile, section_offsets)
-     bfd *abfd;
-     struct objfile *objfile;
-     struct section_offsets *section_offsets;
+som_symtab_read (bfd *abfd, struct objfile *objfile,
+		 struct section_offsets *section_offsets)
 {
   unsigned int number_of_symbols;
   int val, dynamic;
@@ -126,7 +116,7 @@ som_symtab_read (abfd, objfile, section_offsets)
 
      There's nothing in the header which easily allows us to do
      this.  The only reliable way I know of is to check for the
-     existance of a $SHLIB_INFO$ section with a non-zero size.  */
+     existence of a $SHLIB_INFO$ section with a non-zero size.  */
   /* The code below is not a reliable way to check whether an
    * executable is dynamic, so I commented it out - RT
    * shlib_info = bfd_get_section_by_name (objfile->obfd, "$SHLIB_INFO$");
@@ -356,9 +346,7 @@ som_symtab_read (abfd, objfile, section_offsets)
    capability even for files compiled without -g.  */
 
 static void
-som_symfile_read (objfile, mainline)
-     struct objfile *objfile;
-     int mainline;
+som_symfile_read (struct objfile *objfile, int mainline)
 {
   bfd *abfd = objfile->obfd;
   struct cleanup *back_to;
@@ -366,7 +354,7 @@ som_symfile_read (objfile, mainline)
   do_pxdb (symfile_bfd_open (objfile->name));
 
   init_minimal_symbol_collection ();
-  back_to = make_cleanup ((make_cleanup_func) discard_minimal_symbols, 0);
+  back_to = make_cleanup_discard_minimal_symbols ();
 
   /* Read in the import list and the export list.  Currently
      the export list isn't used; the import list is used in
@@ -419,8 +407,7 @@ som_symfile_read (objfile, mainline)
    We reinitialize buildsym, since we may be reading stabs from a SOM file.  */
 
 static void
-som_new_init (ignore)
-     struct objfile *ignore;
+som_new_init (struct objfile *ignore)
 {
   stabsread_new_init ();
   buildsym_new_init ();
@@ -432,8 +419,7 @@ som_new_init (ignore)
    objfile struct from the global list of known objfiles. */
 
 static void
-som_symfile_finish (objfile)
-     struct objfile *objfile;
+som_symfile_finish (struct objfile *objfile)
 {
   if (objfile->sym_stab_info != NULL)
     {
@@ -445,8 +431,7 @@ som_symfile_finish (objfile)
 /* SOM specific initialization routine for reading symbols.  */
 
 static void
-som_symfile_init (objfile)
-     struct objfile *objfile;
+som_symfile_init (struct objfile *objfile)
 {
   /* SOM objects may be reordered, so set OBJF_REORDERED.  If we
      find this causes a significant slowdown in gdb then we could
@@ -460,22 +445,40 @@ som_symfile_init (objfile)
    Plain and simple for now.  */
 
 static void
-som_symfile_offsets (objfile, addrs)
-     struct objfile *objfile;
-     struct section_addr_info *addrs;
+som_symfile_offsets (struct objfile *objfile, struct section_addr_info *addrs)
 {
   int i;
+  CORE_ADDR text_addr;
 
   objfile->num_sections = SECT_OFF_MAX;
   objfile->section_offsets = (struct section_offsets *)
     obstack_alloc (&objfile->psymbol_obstack, SIZEOF_SECTION_OFFSETS);
 
+  /* FIXME: ezannoni 2000-04-20 The section names in SOM are not
+     .text, .data, etc, but $TEXT$, $DATA$,... We should initialize
+     SET_OFF_* from bfd. (See default_symfile_offsets()). But I don't
+     know the correspondence between SOM sections and GDB's idea of
+     section names. So for now we default to what is was before these
+     changes.*/
+  objfile->sect_index_text = 0;
+  objfile->sect_index_data = 1;
+  objfile->sect_index_bss = 2;
+  objfile->sect_index_rodata = 3;
+
   /* First see if we're a shared library.  If so, get the section
      offsets from the library, else get them from addrs.  */
   if (!som_solib_section_offsets (objfile, objfile->section_offsets))
     {
+      /* Note: Here is OK to compare with ".text" because this is the
+         name that gdb itself gives to that section, not the SOM
+         name. */
+      for (i = 0; i < SECT_OFF_MAX && addrs->other[i].name; i++)
+	if (strcmp (addrs->other[i].name, ".text") == 0)
+	  break;
+      text_addr = addrs->other[i].addr;
+
       for (i = 0; i < SECT_OFF_MAX; i++)
-	ANOFFSET (objfile->section_offsets, i) = addrs -> text_addr;
+	(objfile->section_offsets)->offsets[i] = text_addr;
     }
 }
 
@@ -486,8 +489,7 @@ som_symfile_offsets (objfile, addrs)
    with as "loc_indirect" vars.)
    Return value = number of import symbols read in. */
 int
-init_import_symbols (objfile)
-     struct objfile *objfile;
+init_import_symbols (struct objfile *objfile)
 {
   unsigned int import_list;
   unsigned int import_list_size;
@@ -596,7 +598,7 @@ init_import_symbols (objfile)
     }
 
   objfile->import_list_size = import_list_size;
-  free (string_buffer);
+  xfree (string_buffer);
   return import_list_size;
 }
 
@@ -607,8 +609,7 @@ init_import_symbols (objfile)
    with as "loc_indirect" vars.)
    Return value = number of import symbols read in. */
 int
-init_export_symbols (objfile)
-     struct objfile *objfile;
+init_export_symbols (struct objfile *objfile)
 {
   unsigned int export_list;
   unsigned int export_list_size;
@@ -728,7 +729,7 @@ init_export_symbols (objfile)
     }
 
   objfile->export_list_size = export_list_size;
-  free (string_buffer);
+  xfree (string_buffer);
   return export_list_size;
 }
 
@@ -748,7 +749,7 @@ static struct sym_fns som_sym_fns =
 };
 
 void
-_initialize_somread ()
+_initialize_somread (void)
 {
   add_symtab_fns (&som_sym_fns);
 }
