@@ -1,21 +1,23 @@
 /* Include file for stabs debugging format support functions.
-   Copyright 1986-1991, 1992, 1993 Free Software Foundation, Inc.
+   Copyright 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995,
+   1996, 1997, 1999, 2000 Free Software Foundation, Inc.
 
-This file is part of GDB.
+   This file is part of GDB.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
 /* Definitions, prototypes, etc for stabs debugging format support
    functions.
@@ -29,12 +31,6 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 #define	EXTERN extern
 #endif
 
-/* Convert stab register number (from `r' declaration) to a gdb REGNUM.  */
-
-#ifndef STAB_REG_TO_REGNUM
-#define STAB_REG_TO_REGNUM(VALUE) (VALUE)
-#endif
-
 /* Hash table of global symbols whose values are not known yet.
    They are chained thru the SYMBOL_VALUE_CHAIN, since we don't
    have the correct data for that slot yet.
@@ -45,17 +41,17 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
 
 EXTERN struct symbol *global_sym_chain[HASHSIZE];
 
-extern void common_block_start PARAMS ((char *, struct objfile *));
-extern void common_block_end PARAMS ((struct objfile *));
+extern void common_block_start (char *, struct objfile *);
+extern void common_block_end (struct objfile *);
 
 /* Kludge for xcoffread.c */
 
 struct pending_stabs
-{
-  int count;
-  int length;
-  char *stab[1];
-};
+  {
+    int count;
+    int length;
+    char *stab[1];
+  };
 
 EXTERN struct pending_stabs *global_stabs;
 
@@ -85,26 +81,26 @@ EXTERN int previous_stab_code;
    used to match header-file references to their corresponding data.  */
 
 struct header_file
-{
+  {
 
-  /* Name of header file */
-  
-  char *name;
+    /* Name of header file */
 
-  /* Numeric code distinguishing instances of one header file that produced
-     different results when included.  It comes from the N_BINCL or N_EXCL. */
+    char *name;
 
-  int instance;
+    /* Numeric code distinguishing instances of one header file that produced
+       different results when included.  It comes from the N_BINCL or N_EXCL. */
 
-  /* Pointer to vector of types */
+    int instance;
 
-  struct type **vector;
+    /* Pointer to vector of types */
 
-  /* Allocated length (# elts) of that vector */
+    struct type **vector;
 
-  int length;
+    /* Allocated length (# elts) of that vector */
 
-};
+    int length;
+
+  };
 
 /* The table of header_files of this OBJFILE. */
 #define HEADER_FILES(OBJFILE) (DBX_SYMFILE_INFO (OBJFILE)->header_files)
@@ -135,115 +131,92 @@ EXTERN int n_allocated_this_object_header_files;
 extern struct complaint unknown_symtype_complaint;
 extern struct complaint unknown_symchar_complaint;
 
-extern struct type *
-read_type PARAMS ((char **, struct objfile *));
+extern struct type *read_type (char **, struct objfile *);
 
-extern void
-cleanup_undefined_types PARAMS ((void));
+extern void cleanup_undefined_types (void);
 
-extern struct type **
-dbx_lookup_type PARAMS ((int [2]));
+extern struct type **dbx_lookup_type (int[2]);
 
-extern long
-read_number PARAMS ((char **, int));
+extern long read_number (char **, int);
 
-extern void
-add_undefined_type PARAMS ((struct type *));
+extern void add_undefined_type (struct type *);
 
-extern struct symbol *
-define_symbol PARAMS ((CORE_ADDR, char *, int, int, struct objfile *));
+extern struct symbol *define_symbol (CORE_ADDR, char *, int, int,
+				     struct objfile *);
 
-extern void
-stabsread_init PARAMS ((void));
+extern void stabsread_init (void);
 
-extern void
-stabsread_new_init PARAMS ((void));
+extern void stabsread_new_init (void);
 
-extern void
-start_stabs PARAMS ((void));
+extern void start_stabs (void);
 
-extern void
-end_stabs PARAMS ((void));
-
-extern void
-finish_global_stabs PARAMS ((struct objfile *objfile));
+extern void end_stabs (void);
 
+extern void finish_global_stabs (struct objfile *objfile);
 
+
 EXTERN int os9k_stabs;
 
 /* COFF files can have multiple .stab sections, if they are linked
    using --split-by-reloc.  This linked list is used to pass the
    information into the functions in dbxread.c.  */
 struct stab_section_list
-{
-  /* Next in list.  */
-  struct stab_section_list *next;
+  {
+    /* Next in list.  */
+    struct stab_section_list *next;
 
-  /* Stab section.  */
-  asection *section;
-};
+    /* Stab section.  */
+    asection *section;
+  };
 
 /* Functions exported by dbxread.c.  These are not in stabsread.c because
    they are only used by some stabs readers.  */
 
-extern struct partial_symtab *
-start_psymtab PARAMS ((struct objfile *, struct section_offsets *, char *,
-		       CORE_ADDR, int, struct partial_symbol **,
-		       struct partial_symbol **));
-
-extern struct partial_symtab *
-end_psymtab PARAMS ((struct partial_symtab *pst,
-		     char **include_list,
-		     int num_includes,
-		     int capping_symbol_offset,
-		     CORE_ADDR capping_text,
-		     struct partial_symtab **dependency_list,
-		     int number_dependencies,
-		     int textlow_not_set));
+extern struct partial_symtab *end_psymtab (struct partial_symtab *pst,
+					   char **include_list,
+					   int num_includes,
+					   int capping_symbol_offset,
+					   CORE_ADDR capping_text,
+					   struct partial_symtab
+					   **dependency_list,
+					   int number_dependencies,
+					   int textlow_not_set);
 
 extern void
-process_one_symbol PARAMS ((int, int, CORE_ADDR, char *,
-			    struct section_offsets *, struct objfile *));
+process_one_symbol (int, int, CORE_ADDR, char *,
+		    struct section_offsets *, struct objfile *);
 
 extern void elfstab_build_psymtabs
-  PARAMS ((struct objfile *objfile,
-	   struct section_offsets *section_offsets,
-	   int mainline,
-	   file_ptr staboff, unsigned int stabsize,
-	   file_ptr stabstroffset,
-	   unsigned int stabstrsize));
+  (struct objfile *objfile,
+   int mainline,
+   file_ptr staboff, unsigned int stabsize,
+   file_ptr stabstroffset, unsigned int stabstrsize);
 
 extern void coffstab_build_psymtabs
-  PARAMS ((struct objfile *objfile,
-	   struct section_offsets *section_offsets,
-	   int mainline,
-	   CORE_ADDR textaddr, unsigned int textsize,
-	   struct stab_section_list *stabs,
-	   file_ptr stabstroffset,
-	   unsigned int stabstrsize));
+  (struct objfile *objfile,
+   int mainline,
+   CORE_ADDR textaddr, unsigned int textsize,
+   struct stab_section_list *stabs,
+   file_ptr stabstroffset, unsigned int stabstrsize);
 
 extern void stabsect_build_psymtabs
-  PARAMS ((struct objfile *objfile,
-	   struct section_offsets *section_offsets,
-	   int mainline,
-	   char *stab_name,
-	   char *stabstr_name,
-	   char *text_name));
+  (struct objfile *objfile,
+   int mainline, char *stab_name, char *stabstr_name, char *text_name);
 
-extern void elfstab_offset_sections PARAMS ((struct objfile *,
-					     struct partial_symtab *));
+extern void elfstab_offset_sections (struct objfile *,
+				     struct partial_symtab *);
 
-extern void process_later 
-  PARAMS ((struct symbol *, char *, 
-	   int (*f) PARAMS ((struct objfile *, struct symbol *, char *))));
+extern void process_later
+  (struct symbol *, char *,
+   int (*f) (struct objfile *, struct symbol *, char *));
 
-extern int symbol_reference_defined PARAMS ((char **));
+extern int symbol_reference_defined (char **);
 
-extern void ref_add PARAMS ((int, struct symbol *, char *, CORE_ADDR));
+extern void ref_add (int, struct symbol *, char *, CORE_ADDR);
 
-extern struct symbol * ref_search PARAMS ((int));
+extern struct symbol *ref_search (int);
 
-extern int resolve_cfront_continuation 
-  PARAMS ((struct objfile * objfile, struct symbol * sym, char * p));
+extern int resolve_cfront_continuation
+  (struct objfile *objfile, struct symbol *sym, char *p);
 
 #undef EXTERN
